"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initTerraformEnvironment = exports.terraformAwsCli = exports.AWSCloudProvider = void 0;
const infra_aws_1 = require("@goldstack/infra-aws");
const utils_terraform_1 = require("@goldstack/utils-terraform");
const tfState_1 = require("./tfState");
const crypto_1 = __importDefault(require("crypto"));
const child_process_1 = require("child_process");
const os_1 = __importDefault(require("os"));
const getRemoteStateConfig = (config, userName) => {
    const userConfig = config.remoteState.filter((u) => u.user === userName);
    if (userConfig.length === 0) {
        throw new Error(`Cannot find aws user ${userName} in project terraform config`);
    }
    return userConfig[0];
};
class AWSCloudProvider {
    constructor(credentials, awsConfig, region) {
        this.generateEnvVariableString = () => {
            return (`-e AWS_ACCESS_KEY_ID=${this.user.accessKeyId} ` +
                `-e AWS_SECRET_ACCESS_KEY=${this.user.secretAccessKey} ` +
                `-e AWS_SESSION_TOKEN=${this.user.sessionToken || ''} ` +
                `-e AWS_DEFAULT_REGION=${this.region} `);
        };
        this.setEnvVariables = () => {
            process.env.AWS_ACCESS_KEY_ID = this.user.accessKeyId;
            process.env.AWS_SECRET_ACCESS_KEY = this.user.secretAccessKey;
            process.env.AWS_DEFAULT_REGION = this.region;
            process.env.AWS_SESSION_TOKEN = this.user.sessionToken || '';
        };
        this.getTfStateVariables = (deployment) => {
            const remoteStateConfig = getRemoteStateConfig(this.remoteStateConfig, deployment.awsUser);
            const bucket = remoteStateConfig.terraformStateBucket;
            if (!bucket) {
                throw new Error(`TF state bucket not defined for user ${deployment.awsUser}`);
            }
            const ddTable = remoteStateConfig.terraformStateDynamoDBTable;
            if (!ddTable) {
                throw new Error(`TF state DynamoDB table not defined for user ${deployment.awsUser}`);
            }
            const tfKey = deployment.tfStateKey;
            if (!tfKey) {
                throw new Error('Terraform state key not defined');
            }
            return [
                ['bucket', bucket],
                ['key', `${tfKey}`],
                ['region', this.region],
                ['dynamodb_table', ddTable],
            ];
        };
        this.user = credentials;
        this.remoteStateConfig = awsConfig;
        this.region = region;
    }
}
exports.AWSCloudProvider = AWSCloudProvider;
const prompt = (message) => {
    process.stdout.write(message);
    let cmd;
    let args;
    if (os_1.default.platform() == 'win32') {
        cmd = 'cmd';
        args = ['/V:ON', '/C', 'set /p response= && echo !response!'];
    }
    else {
        cmd = 'bash';
        args = ['-c', 'read response; echo "$response"'];
    }
    const opts = {
        stdio: ['inherit', 'pipe', 'inherit'],
        shell: false,
        encoding: 'utf-8',
    };
    return (0, child_process_1.spawnSync)(cmd, args, opts).stdout.toString().trim();
};
const terraformAwsCli = async (args, options) => {
    const { awsTerraformConfig, deployment, credentials, awsProvider } = await initTerraformEnvironment(args);
    const projectHash = crypto_1.default.randomBytes(20).toString('hex');
    const remoteStateConfig = getRemoteStateConfig(awsTerraformConfig, deployment.awsUser);
    if (!remoteStateConfig.terraformStateBucket) {
        remoteStateConfig.terraformStateBucket = `goldstack-tfstate-${projectHash}`;
        (0, infra_aws_1.writeTerraformConfig)(awsTerraformConfig);
    }
    if (!remoteStateConfig.terraformStateDynamoDBTable) {
        remoteStateConfig.terraformStateDynamoDBTable = `goldstack-tfstate-${projectHash}-lock`;
        (0, infra_aws_1.writeTerraformConfig)(awsTerraformConfig);
    }
    const [operation] = args;
    if (operation === 'destroy-state') {
        const ciConfirmed = args.find((str) => str === '-y');
        if (!ciConfirmed) {
            const value = prompt('Are you sure to destroy your remote deployment state? If yes, please type `y` and enter.\n' +
                'Otherwise, press enter.\nYour Input: ');
            if (value !== 'y') {
                new Error('Prompt not confirmed with `y`');
            }
        }
        await (0, tfState_1.deleteState)({
            bucketName: remoteStateConfig.terraformStateBucket,
            dynamoDBTableName: remoteStateConfig.terraformStateDynamoDBTable,
            credentials,
            awsRegion: deployment.awsRegion,
        });
        return;
    }
    await (0, tfState_1.assertState)({
        bucketName: remoteStateConfig.terraformStateBucket,
        dynamoDBTableName: remoteStateConfig.terraformStateDynamoDBTable,
        credentials,
        awsRegion: deployment.awsRegion,
    });
    if (operation === 'create-state') {
        // using this operation, we only create the state and do nothing else.
        return;
    }
    (0, utils_terraform_1.terraformCli)(args, {
        ...options,
        provider: awsProvider,
    });
};
exports.terraformAwsCli = terraformAwsCli;
async function initTerraformEnvironment(args) {
    const deploymentName = args[1];
    const deployment = (0, infra_aws_1.readDeploymentFromPackageConfig)(deploymentName);
    const awsUser = await (0, infra_aws_1.getAWSUser)(deployment.awsUser);
    const credentials = await (0, infra_aws_1.getAWSCredentials)(awsUser);
    const awsTerraformConfig = (0, infra_aws_1.assertTerraformConfig)(deployment.awsUser);
    const awsProvider = new AWSCloudProvider(credentials, awsTerraformConfig, deployment.awsRegion);
    return { awsTerraformConfig, deployment, credentials, awsProvider };
}
exports.initTerraformEnvironment = initTerraformEnvironment;
//# sourceMappingURL=utilsTerraformAws.js.map